function preprocess () {
	echo ('bind.tables <- function (...) {\n');
	echo ('	tables <- list (...)\n');
	echo ('	output <- unlist (tables)\n');
	echo ('	names <- attr(attributes(tables[[1]])$dimnames, "names")\n');
	echo ('	dim (output) <- c (dim (tables[[1]]), length (tables))\n');
	echo ('	dimnames(output) <- list (var1 = dimnames (tables[[1]])[[1]],\n');
	echo ('	  var2=dimnames(tables[[1]])[[2]], statistic=names (tables))\n');
	echo ('	attr(dimnames(output), "names")[1] <- names[1]\n');
	echo ('	attr(dimnames(output), "names")[2] <- names[2]\n');
	echo ('	ftable (output, col.vars=names[2])\n');
	echo ('}\n');
	echo ('\n');
}

function calculate () {
	var x = getValue ("x") ;
	var y = "substitute (" + trim (getValue ("y")).replace (/\n/g, "), substitute (") + ")";

	echo ('x <- ' + x + "\n");
	echo ('yvars <- list (' + y + ')\n');
	echo ('tables <- list()\n');
        echo ('tables.row <- list()\n');
	echo ('tables.column <- list()\n');
	echo ('tables.total <- list()\n');
	echo ('descriptions <- list ()\n');
	echo ('\n');
	echo ('# calculate crosstabs\n');
	echo ('for (i in 1:length (yvars)) {\n');
	echo (' descriptions[[i]] <- list (\'Dependent\'=rk.get.description (' + x + '), \'Independent\'=rk.get.description (yvars[[i]], is.substitute=TRUE))\n');
	echo ('\n');
	echo ('	yvar <- eval (yvars[[i]], envir=globalenv ())\n');
	echo ('	tables[[i]] <- xtabs(~ x + yvar)\n');
	echo ('	names(dimnames(tables[[i]])) <- c(descriptions[[i]][[\'Dependent\']], descriptions[[i]][[\'Independent\']])\n');
	echo ('	tables.row[[i]] <- prop.table(tables[[i]], 1) * 100\n');
	echo (' tables.column[[i]] <- prop.table(tables[[i]], 2) * 100\n');
	echo ('	tables.total[[i]] <- prop.table(tables[[i]]) * 100\n');
	echo ('}\n');
	if (getValue ("chisq") == "TRUE") {
		echo ('\n');
		echo ('# calculate chisquares\n');
		echo ('chisquares <- list ()\n');
		echo ('for (i in 1:length (tables)) {\n');
		echo ('	chisquares[[i]] <- chisq.test (tables[[i]], simulate.p.value = ' + getValue ("simpv"));
		if (getValue ("simpv") == "TRUE") {
			echo (',B=(');
			echo (getValue ("B"));
			echo (') ');
		}
		echo (')\n');
		echo ('}\n');
	}
	echo ('\n');
	echo ('results <- list()\n');
	echo ('for (i in 1:length(tables)) {\n');
	echo ('	results[[i]] <- bind.tables("counts" = tables[[i]]');
	echo (', "% of row" = tables.row[[i]]');
	echo (', "% of column" = tables.column[[i]]');
	echo (', "% of total" = tables.total[[i]]');
	if (getValue("chisq") == "TRUE") {
		echo (', "expected" = chisquares[[i]]$expected');
	}
	echo (')\n');
	echo ('}\n');

}

function printout () {
	doPrintout (true);
}

function preview () {
	calculate ();
	doPrintout (false);
}

function doPrintout (full) {
	if (full) {
		echo ('rk.header ("Crosstabs (n to 1)", level=1)\n');
		echo ('for (i in 1:length (results)) {\n');
		echo ('	rk.header ("Crosstabs (n to 1)", parameters=list ("Dependent", descriptions[[i]][[\'Dependent\']], "Independent", descriptions[[i]][[\'Independent\']]), level=2)\n');
		echo ('	rk.print(results[[i]])');
		if (getValue ("chisq") == "TRUE") {
			echo ('\n');
			echo ('	rk.header ("Pearson\'s Chi Square Test for Crosstabs", list ("Dependent", descriptions[[i]][[\'Dependent\']], "Independent", descriptions[[i]][[\'Independent\']], "Method", chisquares[[i]][["method"]]), level=2)\n');
			echo ('	chisq.show <- as.list(chisquares[[i]][1:3])\n');
			echo ('	names(chisq.show) <- c(names(chisq.show[[1]]), names(chisq.show[[2]]), names(chisq.show)[3])\n');
			echo ('	rk.results (chisq.show)\n');
		}

		if (getValue ("barplot") == "TRUE") {
			echo ('\n');
			echo ('	rk.header ("Barplot for Crosstabs", list ("Dependent", descriptions[[i]][[\'Dependent\']], "Independent", descriptions[[i]][[\'Independent\']]' + getValue ('barplot_embed.code.preprocess') + '), level=2)\n');
			echo ('	rk.graph.on ()\n');
			echo ('	try ({\n');
			printIndented ("\t\t", getValue ('barplot_embed.code.printout'));
			echo ('	})\n');
			echo ('	rk.graph.off ()\n');
		}
		echo ('}\n');
	} else {
		// produce a single barplot of the first result
		echo ("i <- 1\n");
		echo (getValue ('barplot_embed.code.printout'));
	}

}

